''
'' ******************************************************************************
'' *                                                                            *
'' * App Filename: Si570_A01.spin                                               *
'' * Archive Filename:                                                          *
'' *                                                                            *
'' * Description:                                                               *
'' * A Single I2C Si570 demo for the Parallax Propeller and the Parallax 4X20   *
'' * Serial LCD display                                                         *
'' *                                                                            *
'' * Author: D.R., Parallax Propeller Forum username is Drone.                  *
'' * Version: A01 (Alpha), 18 May 2008, Not for distribution.                   *
'' * Parts From: James Burrows "I2C Demo Propeller Program" V2.1 Oct 2007       *
'' * Uses: - Mike Green's "basic_i2c_driver" with "extra bits" from J. Burrows  *
'' *         (Note, I removed the write buffer delays added to Mike's           *
'' *         bsic_i2c_driver as there is no buffer in the Si570 and the         *
'' *         delays may cause a problem with the 10ms write constraint          *
'' *         between unfreezing the Si570 DCO and tickling the NewFreq bit).    *
'' *       - Debug_LCD V1.2                                                     *
'' *                                                                            *
'' * For more info on the Parallax Propeller uC see www.parallax.com.           *
'' * The dependent objects and apps mentioned above may be downloaded from the  *
'' * Object Exchange on the Parallax site.                                      *
'' *                                                                            *
'' ******************************************************************************
''
     
CON

  _xinfreq = 5_000_000                                  ' 5 MHz cyrstal
  _clkmode = xtal1 + pll16x                             ' use crystal x 16 (sys clock = 80 MHz)

  LCDPin   = 27                                         ' for Parallax 4x20 serial LCD on P27
  LCDBaud  = 19_200
  LCDLines = 4
  LCDBlite = true
  LCDCurs  = 3
  
  ' I2C Pin Assignments    
  i2cSCL         = 9
'  i2cSDA         = 10                                  ' info only, do not un-comment pins must be adjacent

  ' Si570 I2C Addr
  DevAddr     = %1010_1010  
  
VAR
  long  i2cAddress, i2cSlaveCounter

OBJ
  i2cObject      : "basic_i2c_driver"
  lcd            : "debug_lcd"  
  
PUB Main

    ' initialize the lcd
    LCD_Init

    ' initialize i2cobject
    i2cObject.Initialize(i2cSCL)
    lcd.gotoxy(4,1)
    lcd.str(string("Initializing"))
    lcd.gotoxy(6,2)
    lcd.str(string("I2C Bus"))
    waitcnt(clkfreq * 1 + cnt)                          ' pause 2 seconds for display
    lcd.cls

    ' i2c bus scan
    i2cScan                                             ' uncomment for I2C bus scan
    waitcnt(clkfreq * 2 + cnt)                          ' uncomment for I2C bus scan

    ' demo the Si570
    if i2cObject.devicePresent(i2cSCL,DevAddr) == true
        lcd.cls
        lcd.str(string("Device Present", 13))
        waitcnt(clkfreq * 1 + cnt)                      ' Wait 1 seconds
        LCD.cls
        Dev_Demo
    else
        lcd.str(string("Device Missing", 13))    
        waitcnt(clkfreq * 1 + cnt)                      ' Wait 1 seconds

PRI LCD_Init

    if lcd.init(LCDPin, LCDBaud, LCDLines)              ' start lcd
    lcd.cls                                             ' clear the lcd
    lcd.cursor(LCDCurs)                                 ' cursor on+blink
    lcd.backLight(LCDBlite)                             ' backlight on (if available)
    
PRI i2cScan | value, ackbit

  ' Scan the I2C Bus and output results to the LCD
  lcd.str(string("Scanning I2C Bus...", 13))

  ' initialize variables
  i2cSlaveCounter := 0

  ' i2cScan - scans 128 addresses on the bus, 0 - 127
  ' sends the address byte and listens for the device to ACK (hold the SDA low)
  repeat i2cAddress from 0 to 127
      value :=  i2cAddress << 1 | 0
      ackbit := i2cObject.devicePresent(i2cSCL, value)
      if ackbit==false
          'debug.str(string("NAK"))
      else
          ' show the scan ACK results 
          lcd.str(string("Addr:%"))    
          lcd.bin(value, 8)
          lcd.str(string(",ACK", 13))                  
          ' the device has set the ACK bit 
          i2cSlaveCounter ++                          
          waitcnt(clkfreq+cnt)                        ' wait 1 sec.
      ' slow the scan so we can read it.    
      waitcnt(clkfreq/4 + cnt)                        ' wait 1/4 sec.

  ' done, display the total number of devices found
  lcd.str(string("Found "))
  lcd.dec(i2cSlaveCounter)
  lcd.str(string(" device(s)!", 13))

PRI Dev_Demo | regAddr, regNum, devData

  ' demo the Si570
  lcd.str(string("Si570 Demo...", 13))                
  waitcnt(clkfreq * 1 + cnt)                          ' wait 1 sec.
  lcd.cls

  regAddr := $600                                     ' location of register 7, should be $7

  ' tell the Si570 to write its configuration to Si570 internal RAM
  i2cObject.WriteByte(i2cSCL, DevAddr, 135, 1)        ' on cold start or reset def vals are there anyway
                                                      ' comment-out to read new reg values if-written
                                                      ' otherwise this is no guarantee writes are working.
                                                      
  ' read and display registers 7 through 12 on LCD
  repeat regNum from 7 to 12
      devData := i2cObject.ReadByte(i2cSCL, DevAddr, regAddr)                
      lcd.str(string("REG"))
      lcd.decx(regNum, 2)
      lcd.str(string(" @ "))
      lcd.ihex(regAddr, 3)
      lcd.str(string(" = "))
      lcd.ihex(devData, 2)
      lcd.str(string(" ", 13))
      regAddr := regAddr + $100                       ' increment the register location
      waitcnt(clkfreq * 2 + cnt)                      ' pause to display
            